﻿Public Class BankAccount
    ' Define the OverdrawnEventArgs type.
    Public Class OverdrawnEventArgs
        Public Property CurrentBalance As Decimal
        Public Property InvalidBalance As Decimal
    End Class

    ' Declare the Overdrawn event.
    Public Event Overdrawn(ByVal sender As Object, ByVal e As OverdrawnEventArgs)

    ' The account balance.
    Public Property Balance As Decimal

    ' Constructors. There is no empty constructor
    ' so the program must use this one.
    Public Sub New(ByVal initialBalance As Decimal)
        If initialBalance < 10 Then
            Throw New ArgumentOutOfRangeException("initialBalance",
                "New BankAccounts must have an initial balance of at least $10.")
        End If

        Balance = initialBalance
    End Sub

    ' Add money to the account.
    Public Sub Credit(ByVal amount As Decimal)
        Balance += amount
    End Sub

    ' Remove money from the account.
    Public Sub Debit(ByVal amount As Decimal)
        ' See if there is enough money.
        If (Balance >= amount) Then
            ' There is enough money. Just do it.
            Balance -= amount
        Else
            ' There's not enough money.
            ' Make an OverdrawnEventArgs to describe the event.
            Dim args As OverdrawnEventArgs = New OverdrawnEventArgs()
            args.CurrentBalance = Balance
            args.InvalidBalance = Balance - amount

            '  Raise the Overdrawn event.
            RaiseEvent Overdrawn(Me, args)
        End If
    End Sub
End Class
