﻿Imports System.Drawing

Public Class Turtle
    ' The TurtleOutOfBoundsEventArgs data type.
    Public Class TurtleOutOfBoundsEventArgs
        ' Where the Turtle would stop if this were not out of bounds.
        Public Property X As Integer
        Public Property Y As Integer
    End Class

    ' Declare the OutOfBounds event.
    Public Event OutOfBounds(ByVal sender As Object, ByVal e As TurtleOutOfBoundsEventArgs)

    ' Properties.
    ' The Bitmap on which we are drawing.
    Public Property Canvas As Bitmap

    ' The Turtle's position.
    Public Property X As Integer
    Public Property Y As Integer

    ' The Turtle's direction in degrees.
    Private _Direction As Integer = 0       ' Backing field.
    Public Property Direction As Integer
        Get
            Return _Direction
        End Get
        Set(ByVal value As Integer)
            'Debug.Assert((value >= 0) AndAlso (value <= 359),
            '    "Direction should be between 0 and 359 degrees")
            _Direction = value
        End Set
    End Property

    '' Store Direction in radians but get and set in degrees.
    'Private _Direction As Double = 0       ' Backing field.
    'Public Property Direction As Integer
    '    Get
    '        Return CInt(_Direction * 180 / Math.PI)
    '    End Get
    '    Set(ByVal value As Integer)
    '        'Debug.Assert((value >= 0) AndAlso (value <= 359),
    '        '    "Direction should be between 0 and 359 degrees")
    '        _Direction = value * Math.PI / 180
    '    End Set
    'End Property

    ' Make the Turtle move the indicated distance in its current direction.
    Public Sub Move(ByVal distance As Integer)
        ' Calculate the new position.
        Dim radians As Double = direction * Math.PI / 180
        Dim newX As Integer = CInt(X + Math.Cos(radians) * distance)
        Dim newY As Integer = CInt(Y + Math.Sin(radians) * distance)

        ' Move to this position.
        Move(newX, newY)
    End Sub

    ' Move to the indicated position.
    Public Sub Move(ByVal newX As Integer, ByVal newY As Integer)
        ' See if the new position is off the Bitmap.
        If ((newX < 0) OrElse (newY < 0) OrElse
            (newX >= Canvas.Width) OrElse (newY >= Canvas.Height)) Then

            ' Raise the OutOfBounds event, passing
            ' the event handler the new coordinates.
            Dim args As TurtleOutOfBoundsEventArgs = New TurtleOutOfBoundsEventArgs()
            args.X = newX
            args.Y = newY
            RaiseEvent OutOfBounds(Me, args)
            Return
        End If

        ' Draw to the new position.
        Using gr As Graphics = Graphics.FromImage(Canvas)
            gr.DrawLine(Pens.Blue, X, Y, newX, newY)
        End Using

        ' Save the new position.
        X = newX
        Y = newY
    End Sub
End Class
