﻿Public Class ComplexNumber
    Public Property Real As Double
    Public Property Imaginary As Double

    ' Constructors.
    Public Sub New(ByVal newReal As Double, ByVal newImaginary As Double)
        Real = newReal
        Imaginary = newImaginary
    End Sub
    Public Sub New(ByVal newReal As Double)
        Real = newReal
        Imaginary = 0
    End Sub

    ' Operators.
    ' Unary -.
    Public Shared Operator -(ByVal c1 As ComplexNumber) As ComplexNumber
        Return New ComplexNumber(-c1.Real, -c1.Imaginary)
    End Operator

    ' Binary +.
    Public Shared Operator +(ByVal c1 As ComplexNumber, ByVal c2 As ComplexNumber) As ComplexNumber
        Return New ComplexNumber(
            c1.Real + c2.Real,
            c1.Imaginary + c2.Imaginary)
    End Operator

    ' Binary *.
    Public Shared Operator *(ByVal c1 As ComplexNumber, ByVal c2 As ComplexNumber) As ComplexNumber
        Return New ComplexNumber(
            c1.Real * c2.Real - c1.Imaginary * c2.Imaginary,
            c1.Real * c2.Imaginary + c1.Imaginary * c2.Real)
    End Operator

    ' Binary -.
    Public Shared Operator -(ByVal c1 As ComplexNumber, ByVal c2 As ComplexNumber) As ComplexNumber
        Return New ComplexNumber(
            c1.Real - c2.Real,
            c1.Imaginary - c2.Imaginary)
    End Operator

    ' Convert between Double and ComplexNumber.
    Public Shared Widening Operator CType(ByVal re As Double) As ComplexNumber
        Return New ComplexNumber(re, 0)
    End Operator
    Public Shared Narrowing Operator CType(ByVal c1 As ComplexNumber) As Double
        Return c1.Magnitude
    End Operator

    ' Return the number's magnitude.
    Public ReadOnly Property Magnitude() As Double
        Get
            Return (Math.Sqrt(Real * Real + Imaginary * Imaginary))
        End Get
    End Property

    ' *******************************************
    ' The following aren't needed for the Try It.
    ' *******************************************
    ' Unary +.
    Public Shared Operator +(ByVal c1 As ComplexNumber) As ComplexNumber
        Return New ComplexNumber(c1.Real, c1.Imaginary)
    End Operator

    ' < and >.
    Public Shared Operator <(ByVal c1 As ComplexNumber, ByVal c2 As ComplexNumber) As Boolean
        Return (c1.Magnitude() < c2.Magnitude())
    End Operator
    Public Shared Operator >(ByVal c1 As ComplexNumber, ByVal c2 As ComplexNumber) As Boolean
        Return (c1.Magnitude() > c2.Magnitude())
    End Operator

    ' <= and >=.
    Public Shared Operator <=(ByVal c1 As ComplexNumber, ByVal c2 As ComplexNumber) As Boolean
        Return (c1.Magnitude() <= c2.Magnitude())
    End Operator
    Public Shared Operator >=(ByVal c1 As ComplexNumber, ByVal c2 As ComplexNumber) As Boolean
        Return (c1.Magnitude() >= c2.Magnitude())
    End Operator

    ' = and <>.
    Public Shared Operator =(ByVal c1 As ComplexNumber, ByVal c2 As ComplexNumber) As Boolean
        Return (c1.Real = c2.Real AndAlso c1.Imaginary = c2.Imaginary)
    End Operator
    Public Shared Operator <>(ByVal c1 As ComplexNumber, ByVal c2 As ComplexNumber) As Boolean
        Return Not (c1 = c2)
    End Operator

    ' Return the number formatted as a string.
    Public Overrides Function ToString() As String
        If Imaginary = 0 Then
            Return String.Format("{0}", Real)
        ElseIf Real = 0 Then
            Return String.Format("{0}i", Imaginary)
        ElseIf Imaginary < 0 Then
            Return String.Format("{0} - {1}i", Real, Math.Abs(Imaginary))
        Else
            Return String.Format("{0} + {1}i", Real, Imaginary)
        End If
    End Function
End Class
