﻿Public Class Form1
    ' The database context.
    Private ScoresContext As TestScoresDatabaseEntities

    ' Display the students in the ComboBox.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Load the data.
        ScoresContext = New TestScoresDatabaseEntities()

        ' Display the students in the ComboBox.
        DisplayStudents()
    End Sub

    ' Display the students in the ComboBox.
    Private Sub DisplayStudents()
        If ScoresContext Is Nothing Then Exit Sub

        If radAllStudents.Checked Then
            Dim allStudentsQuery =
                From stu As Student In ScoresContext.Students
                Order By stu.FirstName, stu.LastName
                Select stu

            ' Bind the Students to the cboStudents ComboBox.
            cboStudents.DataSource = allStudentsQuery.ToArray()
        Else
            Dim failingStudentsQuery =
                From stu As Student In ScoresContext.Students.ToArray()
                Order By stu.FirstName, stu.LastName
                Where stu.Average() < 60
                Select stu

            ' Bind the Students to the cboStudents ComboBox.
            cboStudents.DataSource = failingStudentsQuery.ToArray()
        End If
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ScoresContext.Dispose()
    End Sub

    ' Display this student's test scores.
    Private Sub cboStudents_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboStudents.SelectedIndexChanged
        If cboStudents.SelectedItem Is Nothing Then Exit Sub

        ' Get the selected Student.
        Dim stu As Student =
            DirectCast(cboStudents.SelectedItem, Student)

        ' Display this student's scores.
        dgvScores.DataSource = stu.TestScores

        ' Hide navigation properties.
        dgvScores.Columns("StudentId").Visible = False
        dgvScores.Columns("Student").Visible = False

        ' Size the columns.
        dgvScores.AutoResizeColumns()

        ' Select the first score (if there is one).
        If dgvScores.Rows.Count > 0 Then
            dgvScores.Rows(0).Selected = True
        End If
    End Sub

    ' Save changes to the data.
    Private Sub btnSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSave.Click
        Try
            ScoresContext.SaveChanges()
            MessageBox.Show("Changes Saved", "Changes Saved",
                MessageBoxButtons.OK, MessageBoxIcon.Information)
        Catch ex As Exception
            Dim msg As String = ex.Message
            If ex.InnerException IsNot Nothing Then
                msg &= vbCrLf & vbCrLf & "***** Inner Exception *****" &
                    vbCrLf & ex.InnerException.Message
            End If
            MessageBox.Show(msg)
        End Try
    End Sub

    ' Re-display the students.
    Private Sub radStudents_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radAllStudents.CheckedChanged, radFailingStudents.CheckedChanged
        DisplayStudents()
    End Sub
End Class
